@@ -3,57 +3,17 @@ from typing import Callable, Dict, Tuple
 
 from streamlit import session_state as _state
 
-import streamlit.components.v1 as components
+# import streamlit.components.v1 as components
 from streamlit.components.v1 import components as _components
 
 from pollination_streamlit.interactors import ApiClient
 
-# register callback
-# https://gist.github.com/okld/1a2b2fd2cb9f85fc8c4e92e26c6597d5
-
-
-def _patch_register_widget(register_widget):
-    def wrapper_register_widget(*args, **kwargs):
-        user_key = kwargs.get("user_key", None)
-        callbacks = _state.get("_components_callbacks", None)
-
-        # Check if a callback was registered for that user_key.
-        if user_key and callbacks and user_key in callbacks:
-            callback = callbacks[user_key]
-
-            # Add callback-specific args for the real register_widget function.
-            kwargs["on_change_handler"] = callback[0]
-            kwargs["args"] = callback[1]
-            kwargs["kwargs"] = callback[2]
-
-        # Call the original function with updated kwargs.
-        return register_widget(*args, **kwargs)
-
-    return wrapper_register_widget
-
-
-# Patch function only once.
-if not hasattr(_components.register_widget, "__callbacks_patched__"):
-    setattr(_components.register_widget, "__callbacks_patched__", True)
-    _components.register_widget = _patch_register_widget(
-        _components.register_widget)
-
-
-def register_callback(element_key, callback, *callback_args, **callback_kwargs):
-    # Initialize callbacks store.
-    if "_components_callbacks" not in _state:
-        _state._components_callbacks = {}
-
-    # Register a callback for a given element_key.
-    _state._components_callbacks[element_key] = (
-        callback, callback_args, callback_kwargs)
-# ----
-
+from .components_callbacks import register_callback
 
 # get_host
 parent_dir = os.path.dirname(os.path.abspath(__file__))
 build_dir = os.path.join(parent_dir, "./get_host/frontend/build")
-_get_host = components.declare_component("get_host", path=build_dir)
+_get_host = _components.declare_component("get_host", path=build_dir)
 
 
 def get_host(key='foo'):
@@ -79,10 +39,17 @@ def get_host(key='foo'):
 # get_geometry
 parent_dir = os.path.dirname(os.path.abspath(__file__))
 build_dir = os.path.join(parent_dir, "./get_geometry/frontend/build")
-_get_geometry = components.declare_component("get_geometry", path=build_dir)
+_get_geometry = _components.declare_component("get_geometry", path=build_dir)
 
 
-def get_geometry(key='foo'):
+def get_geometry(
+    key: str = 'foo', *, 
+    label: str = None, 
+    options: dict = None,
+    on_change:  Callable = None,
+    args: Tuple = None,
+    kwargs: Dict = None,
+):
     """Create a new instance of "get_geometry".
 
     Parameters
@@ -91,6 +58,31 @@ def get_geometry(key='foo'):
         An optional key that uniquely identifies this component. If this is
         None, and the component's arguments are changed, the component will
         be re-mounted in the Streamlit frontend and lose its current state.
+    label: str or None
+        A string that will be displayed on the button. Defaults to "Get Geometry"
+    options: dictionary or None
+        A dictionary to show / hide button options. Defaults each option to visible & unselected.
+
+        {
+            "subscribe" : {
+                "show": True or False,
+                "selected": True or False
+            },
+            "selection" : {
+                "show": True or False,
+                "selected": True or False
+            }
+        }
+
+    on_change: Callable or None
+        An optional on_change function that will be called when hbjson input
+        is changed.
+    args: Tuple or None
+        An optional tuple of positional arguments that will be passed to the
+        on_change function.
+    kwargs: dictionary or None
+        An optional dictionary of kwargs that will be passed to the on_change
+        function.
 
     Returns
     -------
@@ -104,8 +96,13 @@ def get_geometry(key='foo'):
         where
             'geometry': List of ladybug geometries as dictionary
     """
+    
+    if on_change is not None:
+        args = args or []
+        kwargs = kwargs or {}
+        register_callback(key, on_change, *args, **kwargs)
 
-    get_geometry = _get_geometry(key=key)
+    get_geometry = _get_geometry(key=key, button_label=label, options_config=options)
 
     return get_geometry
 
@@ -113,11 +110,13 @@ def get_geometry(key='foo'):
 # get_hbjson
 parent_dir = os.path.dirname(os.path.abspath(__file__))
 build_dir = os.path.join(parent_dir, "./get_hbjson/frontend/build")
-_get_hbjson = components.declare_component("get_hbjson", path=build_dir)
+_get_hbjson = _components.declare_component("get_hbjson", path=build_dir)
 
 
 def get_hbjson(
     key: str = 'foo', *,
+    label: str = None,
+    options: Dict = None,
     on_change:  Callable = None,
     args: Tuple = None,
     kwargs: Dict = None,
@@ -130,6 +129,22 @@ def get_hbjson(
         An optional key that uniquely identifies this component. If this is
         None, and the component's arguments are changed, the component will
         be re-mounted in the Streamlit frontend and lose its current state.
+    label: str or None
+        A string that will be displayed on the button. Defaults to "Get Geometry"
+    options: dictionary or None
+        A dictionary to show / hide button options. Defaults each option to visible & unselected.
+
+        {
+            "subscribe" : {
+                "show": True or False,
+                "selected": True or False
+            },
+            "selection" : {
+                "show": True or False,
+                "selected": True or False
+            }
+        }
+
     on_change: Callable or None
         An optional on_change function that will be called when hbjson input
         is changed.
@@ -153,23 +168,29 @@ def get_hbjson(
             'hbjson': hbjson model as dictionary
     """
 
-    get_hbjson = _get_hbjson(key=key)
-
     if on_change is not None:
         args = args or []
         kwargs = kwargs or {}
         register_callback(key, on_change, *args, **kwargs)
 
+    get_hbjson = _get_hbjson(key=key, button_label=label, options_config=options)
+
     return get_hbjson
 
 
 # send_geometry
 parent_dir = os.path.dirname(os.path.abspath(__file__))
 build_dir = os.path.join(parent_dir, "./send_geometry/frontend/build")
-_send_geometry = components.declare_component("send_geometry", path=build_dir)
+_send_geometry = _components.declare_component("send_geometry", path=build_dir)
 
 
-def send_geometry(key='foo', *, geometry={}, option='preview'):
+def send_geometry(
+    key='foo', *, 
+    geometry={}, 
+    label: str = None, 
+    option='preview', 
+    options: dict = None ):
+
     """Create a new instance of "send_geometry".
 
     Parameters
@@ -182,12 +203,24 @@ def send_geometry(key='foo', *, geometry={}, option='preview'):
         A dictionary of ladybug geometries.
     option: 'add' | 'preview' | 'clear' | 'subscribe-preview'
         An option that modifies the action to be taken in the host CAD platform.
+    label: str or None
+        A string that will be displayed on the button. Defaults to "Get Geometry"
+    options: dictionary or None
+        A dictionary to show / hide button options. Defaults to show all.
 
+        {
+            'add', = True or False
+            'delete', = True or False
+            'preview', = True or False
+            'clear', = True or False
+            'subscribe-preview' = True or False
+        }
+    
     Returns
     -------
     """
 
-    send_geometry = _send_geometry(geometry=geometry, key=key, option=option)
+    send_geometry = _send_geometry( key=key, geometry=geometry, option=option, button_label=label, options_config = options)
 
     return send_geometry
 
@@ -195,10 +228,15 @@ def send_geometry(key='foo', *, geometry={}, option='preview'):
 # send_hbjson
 parent_dir = os.path.dirname(os.path.abspath(__file__))
 build_dir = os.path.join(parent_dir, "./send_hbjson/frontend/build")
-_send_hbjson = components.declare_component("send_hbjson", path=build_dir)
+_send_hbjson = _components.declare_component("send_hbjson", path=build_dir)
 
 
-def send_hbjson(key='foo', *, hbjson={}, option='preview'):
+def send_hbjson(key='foo', *, 
+    hbjson={}, 
+    option='preview',
+    label: str = None,
+    options: dict = None):
+
     """Create a new instance of "send_hbjson".
 
     Parameters
@@ -211,12 +249,29 @@ def send_hbjson(key='foo', *, hbjson={}, option='preview'):
         An hbjson model as a dictionary.
     option: 'add' | 'preview' | 'clear' | 'subscribe-preview'
         An option that modifies the action to be taken in the host CAD platform.
+    label: str or None
+        A string that will be displayed on the button. Defaults to "Get Geometry"
+    options: dictionary or None
+        A dictionary to show / hide button options. Defaults to show all.
 
+        {
+            'add', = True or False
+            'delete', = True or False
+            'preview', = True or False
+            'clear', = True or False
+            'subscribe-preview' = True or False
+        }
     Returns
     -------
     """
 
-    send_hbjson = _send_hbjson(hbjson=hbjson, key=key, option=option)
+    send_hbjson = _send_hbjson(
+        key=key,
+        hbjson=hbjson,
+        option=option,
+        button_label=label,
+        options_config=options
+    )
 
     return send_hbjson
 
@@ -224,10 +279,15 @@ def send_hbjson(key='foo', *, hbjson={}, option='preview'):
 # send_results
 parent_dir = os.path.dirname(os.path.abspath(__file__))
 build_dir = os.path.join(parent_dir, "./send_results/frontend/build")
-_send_results = components.declare_component("send_results", path=build_dir)
+_send_results = _components.declare_component("send_results", path=build_dir)
 
 
-def send_results(key='foo', *, geometry={}):
+def send_results(key='foo', *, 
+    results={}, 
+    option='preview', 
+    options=None, 
+    label=None ):
+    
     """Create a new instance of "send_results".
 
     Parameters
@@ -236,15 +296,35 @@ def send_results(key='foo', *, geometry={}):
         An optional key that uniquely identifies this component. If this is
         None, and the component's arguments are changed, the component will
         be re-mounted in the Streamlit frontend and lose its current state.
-    geometry: dictionary
-        If host=='rhino' dictionary of ladybug geometries. 
-        If host=='revit' dictionary with fields: 'results', 'sensorGrids' and 'configs.
+    results: dictionary
+        If host === 'rhino' use new GraphicContainer object
+        If host === 'revit' dictionary with fields: 'results', 'sensorGrids' and 'configs.
+    option: 'add' | 'delete' | 'preview' | 'clear' | 'subscribe-preview'
+        An option that modifies the action to be taken in the host CAD platform.
+    label: str or None
+        A string that will be displayed on the button. Defaults to "Get Geometry"
+    options: dictionary or None
+        A dictionary to show / hide button options. Defaults to show all.
 
+        {
+            'add', = True or False
+            'delete', = True or False
+            'preview', = True or False
+            'clear', = True or False
+            'subscribe-preview' = True or False
+        }
+        
     Returns
     -------
     """
 
-    send_results = _send_results(key=key, geometry=geometry)
+    send_results = _send_results(
+        key=key, 
+        results=results, 
+        option=option, 
+        options_config=options, 
+        button_label=label
+    )
 
     return send_results
 
@@ -252,7 +332,7 @@ def send_results(key='foo', *, geometry={}):
 # auth_user
 parent_dir = os.path.dirname(os.path.abspath(__file__))
 build_dir = os.path.join(parent_dir, "./auth_user/frontend/build")
-_auth_user = components.declare_component("auth_user", path=build_dir)
+_auth_user = _components.declare_component("auth_user", path=build_dir)
 
 
 def auth_user(key='foo', api_client: ApiClient = None):
@@ -291,8 +371,8 @@ def auth_user(key='foo', api_client: ApiClient = None):
     client = api_client.__dict__
 
     auth_user = _auth_user(key=key,
-                           access_token=client["jwt_token"],
-                           api_key=client["api_token"],
+                           access_token=client["_jwt_token"],
+                           api_key=client["_api_token"],
                            base_path=client["_host"])
 
     return auth_user
@@ -301,11 +381,17 @@ def auth_user(key='foo', api_client: ApiClient = None):
 # select_account
 parent_dir = os.path.dirname(os.path.abspath(__file__))
 build_dir = os.path.join(parent_dir, "./select_account/frontend/build")
-_select_account = components.declare_component(
+_select_account = _components.declare_component(
     "select_account", path=build_dir)
 
 
-def select_account(key='foo', api_client: ApiClient = None):
+def select_account(
+    key='foo',
+    api_client: ApiClient = None, *,
+    on_change:  Callable = None,
+    args: Tuple = None,
+    kwargs: Dict = None,
+):
     """Create a new instance of "select_account".
 
     Parameters
@@ -322,7 +408,16 @@ def select_account(key='foo', api_client: ApiClient = None):
         ...
         api_client = get_api_client
         ```
-
+    on_change: Callable or None
+        An optional on_change function that will be called when hbjson input
+        is changed.
+    args: Tuple or None
+        An optional tuple of positional arguments that will be passed to the
+        on_change function.
+    kwargs: dictionary or None
+        An optional dictionary of kwargs that will be passed to the on_change
+        function.
+    
     Returns
     -------
     dict
@@ -356,9 +451,14 @@ def select_account(key='foo', api_client: ApiClient = None):
     """
     client = api_client.__dict__
 
+    if on_change is not None:
+        args = args or []
+        kwargs = kwargs or {}
+        register_callback(key, on_change, *args, **kwargs)
+
     select_account = _select_account(key=key,
-                                     access_token=client["jwt_token"],
-                                     api_key=client["api_token"],
+                                     access_token=client["_jwt_token"],
+                                     api_key=client["_api_token"],
                                      base_path=client["_host"])
 
     return select_account
@@ -367,11 +467,18 @@ def select_account(key='foo', api_client: ApiClient = None):
 # # select_project
 parent_dir = os.path.dirname(os.path.abspath(__file__))
 build_dir = os.path.join(parent_dir, "./select_project/frontend/build")
-_select_project = components.declare_component(
+_select_project = _components.declare_component(
     "select_project", path=build_dir)
 
 
-def select_project(key='foo', api_client: ApiClient = None, *, project_owner=None):
+def select_project(
+    key='foo', 
+    api_client: ApiClient = None, *,
+    project_owner=None,
+    on_change:  Callable = None,
+    args: Tuple = None,
+    kwargs: Dict = None,
+):
     """Create a new instance of "select_project".
 
     Parameters
@@ -390,6 +497,15 @@ def select_project(key='foo', api_client: ApiClient = None, *, project_owner=Non
         ```
     project_owner: str or None
         username of project owner
+    on_change: Callable or None
+        An optional on_change function that will be called when hbjson input
+        is changed.
+    args: Tuple or None
+        An optional tuple of positional arguments that will be passed to the
+        on_change function.
+    kwargs: dictionary or None
+        An optional dictionary of kwargs that will be passed to the on_change
+        function.
 
     Returns
     -------
@@ -441,12 +557,17 @@ def select_project(key='foo', api_client: ApiClient = None, *, project_owner=Non
                     }
     """
 
+    if on_change is not None:
+        args = args or []
+        kwargs = kwargs or {}
+        register_callback(key, on_change, *args, **kwargs)
+    
     client = api_client.__dict__
 
     select_project = _select_project(key=key,
                                      project_owner=project_owner,
-                                     access_token=client["jwt_token"],
-                                     api_key=client["api_token"],
+                                     access_token=client["_jwt_token"],
+                                     api_key=client["_api_token"],
                                      base_path=client["_host"])
 
     return select_project
@@ -455,11 +576,19 @@ def select_project(key='foo', api_client: ApiClient = None, *, project_owner=Non
 # select_recipe
 parent_dir = os.path.dirname(os.path.abspath(__file__))
 build_dir = os.path.join(parent_dir, "./select_recipe/frontend/build")
-_select_recipe = components.declare_component(
+_select_recipe = _components.declare_component(
     "select_recipe", path=build_dir)
 
 
-def select_recipe(key='foo', api_client: ApiClient = None, *, project_name=None, project_owner=None):
+def select_recipe(
+    key='foo', 
+    api_client: ApiClient = None, *, 
+    project_name=None, 
+    project_owner=None,
+    on_change:  Callable = None,
+    args: Tuple = None,
+    kwargs: Dict = None,
+):
     """Create a new instance of "select_recipe".
 
     Parameters
@@ -480,6 +609,15 @@ def select_recipe(key='foo', api_client: ApiClient = None, *, project_name=None,
         project name
     project_owner: str or None
         username of the project owner
+    on_change: Callable or None
+        An optional on_change function that will be called when hbjson input
+        is changed.
+    args: Tuple or None
+        An optional tuple of positional arguments that will be passed to the
+        on_change function.
+    kwargs: dictionary or None
+        An optional dictionary of kwargs that will be passed to the on_change
+        function.
 
     Returns
     -------
@@ -489,14 +627,19 @@ def select_recipe(key='foo', api_client: ApiClient = None, *, project_name=None,
             https://api.pollination.cloud/docs#/Projects/get_project_recipes
 
     """
+    if on_change is not None:
+        args = args or []
+        kwargs = kwargs or {}
+        register_callback(key, on_change, *args, **kwargs)
+
     client = api_client.__dict__
 
     select_recipe = _select_recipe(
         key=key,
         project_name=project_name,
         project_owner=project_owner,
-        access_token=client["jwt_token"],
-        api_key=client["api_token"],
+        access_token=client["_jwt_token"],
+        api_key=client["_api_token"],
         base_path=client["_host"])
 
     return select_recipe
@@ -505,13 +648,18 @@ def select_recipe(key='foo', api_client: ApiClient = None, *, project_name=None,
 # select_study
 parent_dir = os.path.dirname(os.path.abspath(__file__))
 build_dir = os.path.join(parent_dir, "./select_study/frontend/build")
-_select_study = components.declare_component(
+_select_study = _components.declare_component(
     "select_study", path=build_dir)
 
 
-def select_study(key='foo', api_client: ApiClient = None, *,
-                 project_name=None,
-                 project_owner=None):
+def select_study(
+    key='foo', api_client: ApiClient = None, *,
+    project_name=None,
+    project_owner=None,
+    on_change:  Callable = None,
+    args: Tuple = None,
+    kwargs: Dict = None,
+):
     """Create a new instance of "select_study".
 
     Parameters
@@ -532,6 +680,15 @@ def select_study(key='foo', api_client: ApiClient = None, *,
         name of project
     project_owner: str or None
         username of project owner
+    on_change: Callable or None
+        An optional on_change function that will be called when hbjson input
+        is changed.
+    args: Tuple or None
+        An optional tuple of positional arguments that will be passed to the
+        on_change function.
+    kwargs: dictionary or None
+        An optional dictionary of kwargs that will be passed to the on_change
+        function.
 
     Returns
     -------
@@ -540,14 +697,19 @@ def select_study(key='foo', api_client: ApiClient = None, *,
 
             https://api.pollination.cloud/docs#/Jobs/list_jobs
     """
+    if on_change is not None:
+        args = args or []
+        kwargs = kwargs or {}
+        register_callback(key, on_change, *args, **kwargs)
+    
     client = api_client.__dict__
 
     select_study = _select_study(
         key=key,
         project_name=project_name,
         project_owner=project_owner,
-        access_token=client["jwt_token"],
-        api_key=client["api_token"],
+        access_token=client["_jwt_token"],
+        api_key=client["_api_token"],
         base_path=client["_host"])
 
     return select_study
@@ -556,14 +718,19 @@ def select_study(key='foo', api_client: ApiClient = None, *,
 # select_run
 parent_dir = os.path.dirname(os.path.abspath(__file__))
 build_dir = os.path.join(parent_dir, "./select_run/frontend/build")
-_select_run = components.declare_component(
+_select_run = _components.declare_component(
     "select_run", path=build_dir)
 
 
-def select_run(key='foo', api_client: ApiClient = None, *,
-               project_name=None,
-               project_owner=None,
-               job_id=None):
+def select_run(
+    key='foo', api_client: ApiClient = None, *,
+    project_name=None,
+    project_owner=None,
+    job_id=None,
+    on_change:  Callable = None,
+    args: Tuple = None,
+    kwargs: Dict = None
+):
     """Create a new instance of "select_run".
 
     Parameters
@@ -584,7 +751,16 @@ def select_run(key='foo', api_client: ApiClient = None, *,
         username of project owner
     job_id: str or None
         id of study
-
+    on_change: Callable or None
+        An optional on_change function that will be called when hbjson input
+        is changed.
+    args: Tuple or None
+        An optional tuple of positional arguments that will be passed to the
+        on_change function.
+    kwargs: dictionary or None
+        An optional dictionary of kwargs that will be passed to the on_change
+        function.
+        
     Returns
     -------
     dict
@@ -593,6 +769,11 @@ def select_run(key='foo', api_client: ApiClient = None, *,
             https://api.pollination.cloud/docs#/Runs/list_runs
     """
 
+    if on_change is not None:
+        args = args or []
+        kwargs = kwargs or {}
+        register_callback(key, on_change, *args, **kwargs)
+
     client = api_client.__dict__
 
     select_run = _select_run(
@@ -600,8 +781,8 @@ def select_run(key='foo', api_client: ApiClient = None, *,
         project_name=project_name,
         project_owner=project_owner,
         job_id=job_id,
-        access_token=client["jwt_token"],
-        api_key=client["api_token"],
+        access_token=client["_jwt_token"],
+        api_key=client["_api_token"],
         base_path=client["_host"])
 
     return select_run
